@@ -26,17 +26,18 @@ class Cliques(list):
 
 
 class Clique:
-    def __init__(self, c):
+    def __init__(self, c, w=1):
         self.list = c
         # store set for use in inclusion()
         self.set = frozenset(c)
+        self.w = w
 
     def inclusion(self, clique):
         if self.set <= clique.set:
             if len(self.list)>1:
-                return 1 if is_sublist(self.list, clique.list+clique.list) else -1
+                return self.w if is_sublist(self.list, clique.list+clique.list) else -self.w
             else:
-                return 1 if self.list[0]==clique.list[len(clique.list)-1] else -1
+                return self.w if self.list[0]==clique.list[-1] else -self.w
         else:
             return 0
 
@@ -61,18 +62,22 @@ def cliques_by_dim(g, max_dim=None):
     N = len(g.vs)
     sorted_cliques = sorted(oriented_cliques, key=lambda x:len(x)+sum(x)/(N*len(x)))
     clqs = Cliques(len(cliques))
-    for _,cs in groupby(sorted_cliques, key=len):
-        clqs.append([Clique(c) for c in cs])
+    for dim,cs in groupby(sorted_cliques, key=len):
+        if dim == 2:
+            # edges can be weighted
+            clqs.append([Clique(c, np.sqrt(g[c[0],c[1]])) for c in cs])
+        else:
+            clqs.append([Clique(c) for c in cs])
     return clqs
 
 
-def dirac(clqs):
+def dirac(clqs, dtype=float):
     """
     Returns the Dirac matrix.
     """
     N = len(clqs)
     Nd = clqs.total_count
-    D = np.zeros((Nd,Nd), np.int8)
+    D = np.zeros((Nd,Nd), dtype=dtype)
     i_offset = 0
     for i,clqs_i in enumerate(clqs):
         i_size = len(clqs_i)
@@ -136,6 +141,62 @@ def subspace(T, i, clqs, j=None):
         raise Exception('Unsupported arguments')
 
 
+class DifferentialOperators:
+    def __init__(self, clqs):
+        self.clqs = clqs
+        self._D = None
+        self._L = None
+        self._grad = None
+        self._div = None
+        self._curl = None
+        self._cocurl = None
+
+    @property
+    def D(self):
+        if self._D is None:
+            self._D = dirac(self.clqs)
+        return self._D
+
+    @property
+    def L(self):
+        if self._L is None:
+            self._L = self.D @ self.D
+        return self._L
+
+    def L_(self, i):
+        return subspace(self.L, i, self.clqs)
+
+    @property
+    def grad(self):
+        if self._grad is None:
+            self._grad = subspace(self.D, 1, self.clqs, j=0)
+        return self._grad
+
+    @property
+    def div(self):
+        if self._div is None:
+            self._div = subspace(self.D, 0, self.clqs, j=1)
+        return self._div
+
+    @property
+    def curl(self):
+        if self._curl is None:
+            if len(self.clqs) > 2:
+                self._curl = subspace(self.D, 2, self.clqs, j=1)
+            else:
+                self._curl = np.zeros((0, self.clqs.dim(1)))
+        return self._curl
+
+    @property
+    def cocurl(self):
+        if self._cocurl is None:
+            if len(self.clqs) > 2:
+                self._cocurl = subspace(self.D, 1, self.clqs, j=2)
+            else:
+                self._cocurl = np.zeros((self.clqs.dim(1), 0))
+        return self._cocurl
+
+
 def dirac_space(v, i, clqs):
     """
     Embeds \Omega_i into \Omega.
